# -*- coding:utf-8 -*-
# @author :adolf
import torch.nn as nn


class BidirectionalLSTM(nn.Module):

    def __init__(self, nIn, nHidden, nOut):
        super(BidirectionalLSTM, self).__init__()

        self.rnn = nn.LSTM(nIn, nHidden, bidirectional=True)
        self.embedding = nn.Linear(nHidden * 2, nOut)

    def forward(self, input):
        recurrent, _ = self.rnn(input)
        T, b, h = recurrent.size()
        t_rec = recurrent.view(T * b, h)

        output = self.embedding(t_rec)
        output = output.view(T, b, -1)

        return output


class CRNN(nn.Module):

    def __init__(self, imgH, nc, nclass, nh, n_rnn=2, leakyRelu=False):
        super(CRNN, self).__init__()
        assert imgH % 16 == 0, 'imgH has to be a multiple of 16'

        ks = [3, 3, 3, 3, 3, 3, 2]
        ps = [1, 1, 1, 1, 1, 1, 0]
        ss = [1, 1, 1, 1, 1, 1, 1]
        nm = [64, 128, 256, 256, 512, 512, 512]

        cnn = nn.Sequential()

        def convRelu(i, batchNormalization=False):
            nIn = nc if i == 0 else nm[i - 1]
            nOut = nm[i]
            cnn.add_module('conv{0}'.format(i),
                           nn.Conv2d(nIn, nOut, ks[i], ss[i], ps[i]))
            if batchNormalization:
                cnn.add_module('batchnorm{0}'.format(i), nn.BatchNorm2d(nOut))
            if leakyRelu:
                cnn.add_module('relu{0}'.format(i),
                               nn.LeakyReLU(0.2, inplace=True))
            else:
                cnn.add_module('relu{0}'.format(i), nn.ReLU(True))

        convRelu(0)
        cnn.add_module('pooling{0}'.format(0), nn.MaxPool2d(2, 2))  # 64x16x64
        convRelu(1)
        cnn.add_module('pooling{0}'.format(1), nn.MaxPool2d(2, 2))  # 128x8x32
        convRelu(2, True)
        convRelu(3)
        cnn.add_module('pooling{0}'.format(2),
                       nn.MaxPool2d((2, 2), (2, 1), (0, 1)))  # 256x4x16
        convRelu(4, True)
        convRelu(5)
        cnn.add_module('pooling{0}'.format(3),
                       nn.MaxPool2d((2, 2), (2, 1), (0, 1)))  # 512x2x16
        convRelu(6, True)  # 512x1x16

        self.cnn = cnn
        self.rnn = nn.Sequential(
            BidirectionalLSTM(512, nh, nh),
            BidirectionalLSTM(nh, nh, nclass))

    def forward(self, input):
        # conv features
        conv = self.cnn(input)
        b, c, h, w = conv.size()
        assert h == 1  # , "the height of conv must be 1"
        conv = conv.squeeze(2)
        conv = conv.permute(2, 0, 1)  # [w, b, c]

        # rnn features
        output = self.rnn(conv)

        return output



# -*- coding:utf-8 -*-
# @author :adolf
import base64
import cv2
import numpy as np
import os
import torch
import torchvision.transforms as transforms
from rpa_ocr.Identify_English.crnn_model import CRNN


class CRNNInference(object):
    def __init__(self, params):
        general_config = params['GeneralConfig']
        infer_config = params['InferenceConfig']

        self.app_scenes = general_config['app_scenes']

        alphabet = general_config['alphabet']
        self.alphabet_dict = {alphabet[i]: i for i in range(len(alphabet))}
        self.decode_alphabet_dict = {v: k for k, v in self.alphabet_dict.items()}

        self.short_size = general_config['short_size']
        self.verification_length = general_config['length']

        self.device = infer_config['Device']
        self.model_path = os.path.join(infer_config['model_path'],
                                       self.app_scenes + "_verification.pth")

        self.transform = transforms.Compose([transforms.ToTensor()])

        self.model = CRNN(imgH=self.short_size, nc=1, nclass=len(alphabet), nh=256)
        self.init_torch_tensor()
        self.resume()
        self.model.eval()

    def init_torch_tensor(self):
        torch.set_default_tensor_type('torch.FloatTensor')
        if self.device != "cpu":
            torch.set_default_tensor_type('torch.cuda.FloatTensor')

    def resume(self):
        self.model.load_state_dict(torch.load(self.model_path, map_location=self.device), strict=True)
        self.model.to(self.device)

    @staticmethod
    def base64_to_opencv(image_base64):
        img = base64.b64decode(image_base64)
        img_array = np.frombuffer(img, np.uint8)
        img = cv2.imdecode(img_array, cv2.IMREAD_COLOR)
        return img

    def predict(self, image_base64):
        img = self.base64_to_opencv(image_base64)

        imgW = int(img.shape[1] * self.short_size / img.shape[0])
        img = cv2.resize(img, (imgW, self.short_size))

        img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        img = img[:, :, np.newaxis]
        img = self.transform(img)

        img = img.unsqueeze(0)
        with torch.no_grad():
            output = self.model(img)

        output = output.squeeze()

        _, preds = output.max(1)
        preds_list = preds.tolist()

        preds_decode_list = [self.decode_alphabet_dict[i] for i in preds_list]
        res_str = ""

        for i in range(len(preds_decode_list)):
            if i == 0 and preds_decode_list[i] != '-':
                res_str += preds_decode_list[i]
            if preds_decode_list[i] != preds_decode_list[i - 1] and preds_decode_list[i] != '-':
                res_str += preds_decode_list[i]

        if len(res_str) > self.verification_length:
            res_str = res_str[-self.verification_length:]
        return res_str



# -*- coding:utf-8 -*-
# @author :adolf
import os
import yaml

from rpa_ocr.Identify_English.inference import CRNNInference


def ocr_pipeline_main(image_with_base64, scenes):
    config_path = os.path.join('verification_service/config', scenes + '.yaml')
    with open(config_path, 'r') as fp:
        config = yaml.load(fp.read(), Loader=yaml.FullLoader)

    with open(config['_BASE_'], 'r') as fp:
        base_config = yaml.load(fp.read(), Loader=yaml.FullLoader)

    # print(base_config)
    # print(config)
    # for key, value in base_config.items():
    #     base_config[key] = config[key]

    # print(base_config)
    res_str = CRNNInference(base_config).predict(image_with_base64)
    return res_str


if __name__ == '__main__':
    import base64
    import random

    img_path = '/home/shizai/adolf/data/jindie/EHHf.png'
    with open(img_path, 'rb') as f:
        image = f.read()
        image_base64 = str(base64.b64encode(image), 'utf-8')

    ocr_predict = ocr_pipeline_main(image_base64, 'jindie')
    print(ocr_predict)

    img_p = '/home/shizai/adolf/data/jindie/'
    img_list = os.listdir(img_p)
    random.shuffle(img_list)
    total = 200
    positive = 0
    for img_name in img_list[:total]:
        with open(os.path.join(img_p,img_name), 'rb') as f:
            image = f.read()
            image_base64 = str(base64.b64encode(image), 'utf-8')

        ocr_predict = ocr_pipeline_main(image_base64, 'jindie')
        # print(ocr_predict)
        # print(img_name)
        # print('---------')
        if ocr_predict == img_name.split('.')[0]:
            positive += 1
        else:
            print(img_name)
            print(ocr_predict)

    print('accuracy', positive / total)
